@@ -10,8 +10,7 @@
 import inspect
 import struct
 import re
-import time
-from .katcp import Message, FailReply, KatcpSyntaxError
+from .katcp import Message, FailReply
 
 # KATCP Type Classes
 #
@@ -467,7 +466,7 @@ class Parameter(object):
 ## request, return_reply and inform method decorators
 #
 
-def request(*types):
+def request(*types, **options):
     """Decorator for request handler methods.
 
     The method being decorated should take a sock argument followed
@@ -488,35 +487,47 @@ def request(*types):
     ...         return ("ok", my_int + 1, my_float / 2.0)
     ...
     """
+    include_msg = options.get('include_msg', False)
+
     def decorator(handler):
         argnames = []
-        orig_argnames = getattr(handler, "_orig_argnames", None)
 
-        if orig_argnames is not None:
-            # If this decorator is on the outside, get the parameter names which have been preserved by the other decorator
-            argnames = orig_argnames
-            # and the sock flag
-            has_sock = getattr(handler, "_has_sock")
-        else:
-            # Introspect the parameter names.
-            # Slightly hacky way of determining whether there is a sock
-            has_sock = inspect.getargspec(handler)[0][1] == "sock"
-            params_start = 2 if has_sock else 1
-            # Get other parameter names
-            argnames = inspect.getargspec(handler)[0][params_start:]
+        # If this decorator is on the outside, get the parameter names which have been preserved by the other decorator
+        all_argnames = getattr(handler, "_orig_argnames", None)
+
+        if all_argnames is None:
+            # We must be on the inside. Introspect the parameter names.
+            all_argnames = inspect.getargspec(handler)[0]
+
+        # Slightly hacky way of determining whether there is a sock
+        has_sock = all_argnames[1] == "sock"
+
+        params_start = 1
+        if has_sock: params_start += 1
+        if include_msg: params_start += 1
+        # Get other parameter names
+        argnames = all_argnames[params_start:]
 
         def raw_handler(self, *args):
             if has_sock:
                 (sock, msg) = args
                 new_args = unpack_types(types, msg.arguments, argnames)
-                return handler(self, sock, *new_args)
+                if include_msg:
+                    return handler(self, sock, msg, *new_args)
+                else:
+                    return handler(self, sock, *new_args)
             else:
                 (msg,) = args
                 new_args = unpack_types(types, msg.arguments, argnames)
-                return handler(self, *new_args)
+                if include_msg:
+                    return handler(self, msg, *new_args)
+                else:
+                    return handler(self, *new_args)
 
         raw_handler.__name__ = handler.__name__
         raw_handler.__doc__ = handler.__doc__
+        # explicitly note that this decorator has been run, so that return_reply can know if it's on the outside.
+        raw_handler._request_decorated = True
         return raw_handler
 
     return decorator
@@ -562,41 +573,31 @@ def return_reply(*types):
             return make_reply(msgname, types, reply_args)
         raw_handler.__name__ = handler.__name__
         raw_handler.__doc__ = handler.__doc__
-        try:
-            # We must preserve the original function parameter names for the other decorator in case this decorator is on the inside
-            # Slightly hacky way of determining whether there is a sock
-            has_sock = inspect.getargspec(handler)[0][1] == "sock"
-            params_start = 2 if has_sock else 1
-            # Get other parameter names
-            raw_handler._orig_argnames = inspect.getargspec(handler)[0][params_start:]
-            # we must also note whether there is a sock
-            raw_handler._has_sock = has_sock
-        except IndexError:
-            # This probably means that this decorator is on the outside.
-            pass
+
+        if not getattr(handler, "_request_decorated", False):
+            # We are on the inside.
+            # We must preserve the original function parameter names for the request decorator
+            raw_handler._orig_argnames = inspect.getargspec(handler)[0]
+
         return raw_handler
 
     return decorator
 
-def send_reply(msgname, *types):
+def send_reply(*types):
     """Decorator for sending replies from request callback methods
 
     This decorator constructs a reply from a list or tuple returned
     from a callback method, but unlike the return_reply decorator it
-    also sends the reply rather than returning it.  The message name
-    must be passed in explicitly, since the callback method is not
-    expected to have a predictable name or input parameters.
+    also sends the reply rather than returning it.
 
     The list/tuple returned from the callback method must have a sock
-    as its first parameter.
+    as its first parameter and the original message as the second. The
+    original message is needed to determine the message name and ID.
 
-    The device with the callback method must have a send_message
-    method.
+    The device with the callback method must have a reply method.
 
     Parameters
     ----------
-    msgname : str
-        Name of the reply message.
     types : list of kattypes
         The types of the reply message parameters (in order).
 
@@ -604,16 +605,17 @@ def send_reply(msgname, *types):
     --------
     >>> class MyDevice(DeviceServer):
     ...     @send_reply('myreq', Int(), Float())
-    ...     def my_callback(self, sock):
-    ...         return (sock, "ok", 5, 2.0)
+    ...     def my_callback(self, msg, sock):
+    ...         return (sock, msg, "ok", 5, 2.0)
     ...
     """
     def decorator(handler):
         def raw_handler(self, *args):
             reply_args = handler(self, *args)
             sock = reply_args[0]
-            reply = make_reply(msgname, types, reply_args[1:])
-            self.send_message(sock, reply)
+            msg = reply_args[1]
+            reply = make_reply(msg.name, types, reply_args[2:])
+            self.reply(sock, reply, msg)
         return raw_handler
 
     return decorator
