@@ -409,15 +409,17 @@ class TestDevice(object):
         if i == 6:
             return ("ok", i, d, b, "extra parameter")
         if i == 9:
-            self.finish_request_one(sock, i, d, b)
+            # This actually gets put in the callback params automatically
+            orig_msg = Message.request("one", "foo", "bar")
+            self.finish_request_one(orig_msg, sock, i, d, b)
             raise AsyncReply()
         return ("ok", i, d, b)
 
-    @send_reply("one", Int(min=1,max=10),Discrete(("on","off")),Bool())
-    def finish_request_one(self, sock, i, d, b):
-        return (sock, "ok", i, d, b)
+    @send_reply(Int(min=1,max=10),Discrete(("on","off")),Bool())
+    def finish_request_one(self, msg, sock, i, d, b):
+        return (sock, msg, "ok", i, d, b)
 
-    def send_message(self, sock, msg):
+    def reply(self, sock, msg, orig_msg):
         self.sent_messages.append([sock, msg])
 
     @request(Int(min=1,max=3,default=2), Discrete(("on","off"),default="off"), Bool(default=True))
@@ -449,6 +451,16 @@ class TestDevice(object):
     def request_six(self, i, d, b):
         return ("ok", i, d, b)
 
+    @return_reply(Int(), Str())
+    @request(Int(), include_msg=True)
+    def request_seven(self, msg, i):
+        return ("ok", i, msg.name)
+
+    @return_reply(Int(), Str())
+    @request(Int(), include_msg=True)
+    def request_eight(self, sock, msg, i):
+        return ("ok", i, msg.name)
+
 
 class TestDecorator(unittest.TestCase):
     def setUp(self):
@@ -524,3 +536,12 @@ class TestDecorator(unittest.TestCase):
         self.assertRaises(FailReply, self.device.request_six, Message.request("six", "2", "on", "3"))
 
         self.assertRaises(FailReply, self.device.request_six, Message.request("six", "2", "on"))
+
+    def test_request_seven(self):
+        """Test client request with no sock but with a message."""
+        self.assertEqual(str(self.device.request_seven(Message.request("seven", "7"))), "!seven ok 7 seven")
+
+    def test_request_eight(self):
+        """Test server request with a message argument."""
+        sock = ""
+        self.assertEqual(str(self.device.request_eight(sock, Message.request("eight", "8"))), "!eight ok 8 eight")
